﻿using System.Collections.Generic;
using UnityEngine;

namespace Magify
{
    public partial class MagifyManager
    {
        public static class Content
        {
            private static readonly MagifyLogger _logger = MagifyLogger.Get(LoggingScope.Content);

            public static List<ContentItem> GetContentList(string group, string key, List<string> tags = null)
            {
                ThrowIfMagifyIsNotReady($"{nameof(Content)}.{nameof(GetContentList)}");
                _logger.Log($"Get content list. {nameof(group)}={group}; {nameof(key)}={key}; {nameof(tags)}={JsonFacade.SerializeObject(tags)}");
                var result = MagifyPlatformAPI.GetContentList(group, key, tags);
                _logger.Log($"Get content list result: {JsonFacade.SerializeObject(result)}");
                return result;
            }

            public static ContentItem GetEarliestContent(string group, string key)
            {
                ThrowIfMagifyIsNotReady($"{nameof(Content)}.{nameof(GetEarliestContent)}");
                _logger.Log($"Get earliest content. {nameof(group)}={group}; {nameof(key)}={key};");
                var result = MagifyPlatformAPI.GetEarliestContent(group, key);
                _logger.Log($"Get earliest content: {JsonFacade.SerializeObject(result)}");
                return result;
            }

            public static ContentItem GetLatestContent(string group, string key)
            {
                ThrowIfMagifyIsNotReady($"{nameof(Content)}.{nameof(GetLatestContent)}");
                _logger.Log($"Get latest content. {nameof(group)}={group}; {nameof(key)}={key};");
                var result = MagifyPlatformAPI.GetLatestContent(group, key);
                _logger.Log($"Get latest content: {JsonFacade.SerializeObject(result)}");
                return result;
            }
        }
    }
}