using System;
using System.Threading;
using System.Threading.Tasks;

namespace Magify
{
    internal static class TaskHelper
    {
        internal static Task WaitUntil(Func<bool> predicate, CancellationToken token = default)
        {
            if (token.IsCancellationRequested)
            {
                return Task.FromCanceled(token);
            }

            var promise = new TaskCompletionSource<object>();
            token.Register(() => promise.TrySetCanceled());
            routine();

            async void routine()
            {
                while (true)
                {
                    if (token.IsCancellationRequested)
                    {
                        promise.TrySetCanceled(token);
                        return;
                    }

                    if (predicate())
                    {
                        promise.SetResult(null);
                        return;
                    }

                    await Task.Yield();
                }
            }

            return promise.Task;
        }
    }
}